# $Header: /usr/local/CVS/Perl-Metrics-Simple/Build.PL,v 1.6 2006/10/03 03:53:08 matisse Exp $
# $Revision: 1.6 $
# $Author: matisse $
# $Source: /usr/local/CVS/Perl-Metrics-Simple/Build.PL,v $
# $Date: 2006/10/03 03:53:08 $
###############################################################################

use Module::Build;

# See perldoc Module::Build for details of how this works

Module::Build->new(
    module_name => 'Perl::Metrics::Simple',
    license     => 'perl',
    requires    => {
        'Readonly'            => 1.03,
        'PPI'                 => 1.113,    
    },
    build_requires => { 'Module::Build' => 0.2612, },
)->create_build_script;
